#ifndef VLC_OBJECT_LOADER_H
#define VLC_OBJECT_LOADER_H

#include <float.h>

#include <vlc_common.h>
#include <vlc_image.h>
#include <vlc_url.h>


typedef struct object_loader_t object_loader_t;
typedef struct object_loader_sys_t object_loader_sys_t;


typedef struct
{
    float transformMatrix[16];
    unsigned meshId;
    unsigned textureId;
} scene_object_t;


typedef struct
{
    float *vCoords;
    float *tCoords;
    unsigned *faces;

    unsigned nVertices;
    unsigned nFaces;
} scene_mesh_t;


typedef struct
{
    char *psz_path;
    picture_t *p_pic;
} scene_texture_t;


typedef struct
{
    scene_object_t **objects;
    scene_mesh_t **meshes;
    scene_texture_t **textures;

    float transformMatrix[16];

    unsigned nObjects;
    unsigned nMeshes;
    unsigned nTextures;
} scene_t;


struct object_loader_t
{
    struct vlc_common_members obj;

    /* Module */
    module_t* p_module;

    /* Private structure */
    object_loader_sys_t* p_sys;

    scene_t * (*loadScene)(object_loader_t *p_loader, const char *psz_path);
};


VLC_API object_loader_t *objLoader_get(vlc_object_t *p_parent);
VLC_API void objLoader_release(object_loader_t* p_objLoader);
VLC_API scene_t *objLoader_loadScene(object_loader_t *p_objLoader, const char *psz_path);
VLC_API scene_object_t *scene_object_New(float *transformMatrix, unsigned meshId,
                                         unsigned textureId);
VLC_API void scene_object_Release(scene_object_t *p_object);
VLC_API scene_mesh_t *scene_mesh_New(unsigned nVertices, unsigned nFaces,
                                     float *vCoords, float *nCoords,
                                     unsigned int *faces);
VLC_API void scene_mesh_Release(scene_mesh_t *p_mesh);
VLC_API scene_texture_t *scene_texture_New(object_loader_t *p_loader, const char *psz_path);
VLC_API void scene_texture_Release(scene_texture_t *p_texture);
VLC_API void scene_CalcTransformationMatrix(scene_t *p_scene);
VLC_API scene_t *scene_New(unsigned nObjects, unsigned nMeshes, unsigned nTextures);
VLC_API void scene_Release(scene_t *p_scene);


#endif // VLC_OBJECT_LOADER_H
